% Function used to obtain bootstrap CIs on Ftheta and C'
function res = estim_sequence(data, grille_theta_bis, E, tau_L)

%% Data preparation

% Interviewers there in (2001 or 2002) and 2003
stayers = data((data(:,2)>0 & data(:,4)>0),:);
% Interviewers there only in 2003
movers03 = data((data(:,2)==0 & data(:,4)>0),:);

n1_st = stayers(:,2);
k1_st = stayers(:,1);

n2_st = stayers(:,4);
k2_st = stayers(:,3);

n2_mov = movers03(:,2);
k2_mov = movers03(:,1);


%%
% Nonparametric sieve MLE estimation of F_{y_1} and F_{y_2} for stayers 
options = optimset('Display','off','MaxFunEvals',1e6,'MaxIter',100000);
grille = (0:1/999:1)';

a0 = sqrt(3)*[0;0;1;0];
a1 = fmincon(@(a) log_vrais_binom(a, k1_st, n1_st), a0, [],[],[],[],[],[],@contrainte_densite,options);

b1 = conv(a1,a1);
b1_int_st =  polyint(b1')';
F_y1_st = polyval(b1_int_st, grille);

a2 = fmincon(@(a) log_vrais_binom(a, k2_st, n2_st), a0, [],[],[],[],[],[],@contrainte_densite,options);
b2 = conv(a2,a2);
b2_int_st =  polyint(b2')';
F_y2_st = polyval(b2_int_st, grille);

% Nonparametric sieve MLE estimation of F_{y_2|mov} 
a0 =  sqrt(3)*[0;1;0];
a2 = fmincon(@(a) log_vrais_binom(a, k2_mov, n2_mov), a0, [],[],[],[],[],[],@contrainte_densite,options);
b2 = conv(a2,a2);
b2_int_mov =  polyint(b2')';

%%
%  Nonparametric estimates of F_theta(theta_n) (stayers) and C'(y_n)

% First, computation of the sequences (y_n) (in decreasing order) and (theta_n)
% (in increasing order).

% Initial points (normalization)
theta0 =  1 ;
% y0 is approximately the median of F_{y1}
y0 = 0.8;


borne_inf= interp1q(F_y2_st,grille,tau_L);
borne_sup = interp1q(F_y1_st,grille,1-tau_L);

grid_y = y0;

y_temp = y0;
y_temp_prec = y0 - 0.3;
k=0;

while y_temp < borne_sup && y_temp > y_temp_prec + 1e-4 && k > -50
    fdr_temp = polyval(b1_int_st, y_temp);
    p = [b2_int_st(1:end-1)' b2_int_st(end)-fdr_temp];
    y_temp_prec = y_temp;
    y_temp = sum(roots(p).* (imag(roots(p))==0) .* (roots(p)<1) .* (roots(p)>0));
    grid_y=[y_temp;grid_y];
    k=k-1;
end


fdr_temp = polyval(b1_int_st, y_temp);
p = [b2_int_st(1:end-1)' b2_int_st(end)-fdr_temp];
hors_grid = sum(roots(p).* (imag(roots(p))==0) .* (roots(p)<1) .* (roots(p)>0));

nb_g = -k;

y_temp = y0;
y_temp_prec = y0 + 0.3;

k=0;

while y_temp > borne_inf  && y_temp + 1e-4 < y_temp_prec && k < 50
    fdr_temp = polyval(b2_int_st, y_temp);
    p = [b1_int_st(1:end-1)' b1_int_st(end)-fdr_temp];
    y_temp_prec = y_temp;
    y_temp = sum(roots(p).* (imag(roots(p))==0) .* (roots(p)<1) .* (roots(p)>0));
    grid_y=[grid_y;y_temp];
    k=k+1;
end



nb_d = k;
% The grid of (theta_k) is simple since theta_k = E^k theta0
grille_theta = theta0 * (E.^((nb_g:-1:-nb_d)'));

taille_tot = nb_g + nb_d + 1;

% grille_theta = theta0 * (E.^((nb_g:-1:-nb_d)'));

temp = grille_theta * ones(1,size(grille_theta_bis,2)) ; 
indic1 = (temp <= ones(taille_tot ,1)* grille_theta_bis);
indic2 = (temp >= ones(taille_tot ,1)* grille_theta_bis);
ind1 =  1 + sum(indic1)' ;
ind2 =  1+ taille_tot  - sum(indic2)' ;
clear temp;

% Nonparametric estimates of F_theta(theta_n) and C'(y_n)

Ftheta_st = 1-polyval(b1_int_st, grid_y);
Ftheta_mov2 = 1-polyval(b2_int_mov,[hors_grid;grid_y(1:end-1)]);
p_st = 1/(1+length(n2_mov)/length(n2_st));

Ftheta = p_st * Ftheta_st + (1-p_st) * Ftheta_mov2 ;

Fth_bis = [0;Ftheta];
Fth_ter = [Ftheta;1];

Ftheta_inf = Fth_bis(ind1)';
Ftheta_sup = Fth_ter(ind2)';

y_bis = [grid_y;0];
y_ter = [1;grid_y];

yinf = y_bis(ind2)';
ysup = y_ter(ind1)';

res = [yinf, ysup, Ftheta_inf, Ftheta_sup];

end