function [param, mat_variance] = estimation_param(data, nb_param_theta, theta0, y0, delta1, delta2, n_s, indic_Ftheta, indic_Cprime)

%% Parametric estimation of C' and F_theta

param = zeros(nb_param_theta+1,1);

vect_unif = halton(n_s,3)';

if nb_param_theta == 4
    % Unconstrained estimation
    vect_param_init = [-0.0634;0.0221;0.1059;-0.8161;0.1442];
else
    % Constrained estimation
    vect_param_init = [-0.0634;0.06;-0.8161;0.1442];
end

options = optimset('GradObj','off','LargeScale','off');

[temp_mle,~,~,~,~, hessian] = fminunc(@(x) log_vrais_parametrique(x, nb_param_theta, theta0, y0, delta1, delta2, vect_unif, data, ...
                                        indic_Ftheta, indic_Cprime), vect_param_init, options);
                                    
param = exp(temp_mle);
if indic_Ftheta==2 
    param(1:nb_param_theta-1)=temp_mle(1:nb_param_theta-1);
end

if nargout > 1

    mat_temp = inv(hessian)/size(data,1);
    
    Jac = diag(param(1:end-1));
        
    if indic_Ftheta == 2
        Jac = diag([ones(nb_param_theta-1,1);param(nb_param_theta:nb_param_theta+1)]);
    end

    mat_variance=Jac * mat_temp * Jac; 
    
end

end
