% Estimation of the parametric model, together with surplus under complete
% information, optimal nonlinear contracts with incomplete information and
% linear contracts

% indic_Cprime =1: phi(x)=ln(x+1), 2:phi(x)=x, 3: phi(x)=exp(x)-1
% indic_Ftheta =1: Frechet, 2: lognormal, 3: Weibul.

function [res, t_opt_asym] = estimation_surplus_et_al(data, vect_param, ...
    theta0, y0, delta1, delta2, n_s, indic_Ftheta, indic_Cprime, t_init, indic_surplus)

vect_unif = halton(n_s,7)';

nb_param_theta = length(vect_param)-1;

a_st = vect_param(1);
if nb_param_theta == 3
    a_mov = vect_param(2);
else
    a_mov1 = vect_param(2);
    a_mov2 = vect_param(3);
end
b = vect_param(end-1);
beta = vect_param(end);

if indic_Cprime==1 
    alpha = (delta1/theta0) / (log(1/(1-y0)).^beta);
elseif indic_Cprime==2
    alpha = (delta1/theta0) / ((y0/(1-y0)).^beta);
else
    alpha = (delta1/theta0) / ((exp(y0/(1-y0))-1).^beta);
end

%% Estimation of lambda1 and lambda2

% Interviewers there in (2001 or 2002) and 2003
stayers = data((data(:,2)>0 & data(:,4)>0),:);

n1moy_st = mean(stayers(:,2) );
n2moy_st = mean(stayers(:,4));
n1moy_mov = mean(data((data(:,4)==0),2));
n2moy_mov = mean(data((data(:,2)==0),4));

p_st1 = size(stayers,1)/sum((data(:,2)>0));
p_st2 = size(stayers,1)/sum((data(:,4)>0));
n_st1= floor(p_st1*n_s);
n_st2= floor(p_st2*n_s);

n1moy = p_st1 * n1moy_st + (1- p_st1) * n1moy_mov ;
n2moy = p_st2 * n2moy_st + (1- p_st2) * n2moy_mov ;


poids_st1 = p_st1 * n1moy_st /n1moy;
poids_st2 = p_st2 * n2moy_st /n2moy;


vect_theta_st1 = inv_Ftheta([a_st;b], vect_unif(1:n_st1), indic_Ftheta)';
vect_theta_st2 = inv_Ftheta([a_st;b], vect_unif(1:n_st2), indic_Ftheta)';

if nb_param_theta==3
    vect_theta_mov1 = inv_Ftheta([a_mov;b], vect_unif(n_st1+1:end), indic_Ftheta)';
    vect_theta_mov2 = inv_Ftheta([a_mov;b], vect_unif(n_st2+1:end), indic_Ftheta)';
else
    vect_theta_mov1 = inv_Ftheta([a_mov1;b], vect_unif(n_st1+1:end), indic_Ftheta)';
    vect_theta_mov2 = inv_Ftheta([a_mov2;b], vect_unif(n_st2+1:end), indic_Ftheta)';
end
    
vect_theta1 = [vect_theta_st1;vect_theta_mov1];
vect_theta2 = [vect_theta_st2;vect_theta_mov2];

y1 = inv_Cprime(beta, theta0, y0, delta1, delta1./vect_theta1, indic_Cprime) ;
y2 = inv_Cprime(beta, theta0, y0, delta1, delta2./vect_theta2, indic_Cprime) ;

temp_deriv1 = (delta1./(alpha* vect_theta1)).^(1/beta -1 )./(alpha * beta * vect_theta1);
temp_deriv2 = (delta2./(alpha* vect_theta2)).^(1/beta -1 )./(alpha * beta * vect_theta2);

if indic_Cprime == 1 
    dyddelta1 = (1-y1) .* temp_deriv1 ;
    dyddelta2 = (1-y2) .* temp_deriv2 ;
elseif indic_Cprime == 2 
    dyddelta1 = y1.*(1-y1)/(beta*delta1);
    dyddelta2 = y2.*(1-y2)/(beta*delta2);
else
    dyddelta1 = (1-y1).^2 .* temp_deriv1 ./ (1+((delta1./(alpha * vect_theta1)).^(1/beta)));
    dyddelta2 = (1-y2).^2 .* temp_deriv2 ./ (1+((delta2./(alpha * vect_theta2)).^(1/beta)));
end

lambda1 = delta1 +  (poids_st1 * mean(y1(1:n_st1)) + (1-poids_st1) * mean(y1(n_st1+1:end)))...
           /(poids_st1 * mean(dyddelta1(1:n_st1)) + (1-poids_st1) * mean(dyddelta1(n_st1+1:end)));
lambda2 = delta2 +  (poids_st2 * mean(y2(1:n_st2)) + (1-poids_st2) * mean(y2(n_st2+1:end)))...
           /(poids_st2 * mean(dyddelta2(1:n_st2)) + (1-poids_st2) * mean(dyddelta2(n_st2+1:end)));

res=[alpha; lambda1; lambda2];

if nargout>1
   t_opt_asym=0;
end
    

if indic_surplus==1
    %% Current surplus                                        

    surplus_actuel = (lambda2  - delta2) * (poids_st2 * mean(y2(1:n_st2)) + ... 
                      (1-poids_st2) *mean(y2(n_st2+1:end))) -4.5;

%     %% Surplus with optimal quadratic contract
%     
%     [a_b_optim, surplus_quad] = t_opt_quadr(alpha,beta,lambda2,mat_trie, data, indic_Cprime);
% 
%     ratio_quad = surplus_actuel / surplus_quad 

    %% Surplus with optimal asymmetric contract

    pr_prop = tabulate(data((data(:,4)>0),4));
    pr_prop = pr_prop((pr_prop(:,2)>0),:);
    prop_n = pr_prop(:,3)/100;
    liste_n = pr_prop(:,1);
    nb_n = length(liste_n);

    vect_surplus = zeros(nb_n,1);
    y_opt_asym = zeros(nb_n,1);
    tstar = zeros(liste_n(end)+1,nb_n);
    prop_gain = zeros(nb_n,1);

    % We have to compute Pr(P=p|n). We use for that purpose a logit model
    % Pr(P=p|n) = 1/(1+exp[-(a0 + a1 * n + a2 * n * 1(n>=10))]

    n2 = data((data(:,4)>0),4);
    indic_st=(data((data(:,4)>0),2)>0);
    beta_logit = glmfit(n2,indic_st,'binomial');
    proba_st_n = 1./(1+exp(-[ones(nb_n,1), liste_n]*beta_logit));

    mat_trie=sortrows([vect_theta2,[ones(n_st2,1);zeros(n_s-n_st2,1)]])';
    theta_trie = mat_trie(1,:);
    indic_stayers=mat_trie(2,:);
    y2_trie = sort(y2,'descend')';
    util_lin = delta2 * y2_trie  - alpha * theta_trie .* C_prime(y2_trie,...
                        beta, indic_Cprime);
    
    for k=1:nb_n
        n = liste_n(k);
        % This vector assigns weights  Pr(P=p|n)
        vect_weight = ((proba_st_n(k)/p_st2) * indic_stayers ...
            + (1- proba_st_n(k))/(1-p_st2) * (1-indic_stayers))' ;
        vect_weight = vect_weight/sum(vect_weight);
        if k<4
            if isempty(t_init)
                if k==1
                    t0=[10;28];
                else
                    n_prec = liste_n(k-1);
                    t0 = interp1((0:1/n_prec:1)',tstar(1:n_prec+1,k-1),(1/n:1/n:1)');
                end
            else
                t0 = t_init(2:n+1,k);
            end
            [temp, vect_surplus(k)] = t_opt(alpha,beta,lambda2,n,theta_trie, vect_weight,t0,indic_Cprime);
            tstar(1:n+1,k) = temp;
            vect_y_asym = y_theta_t(theta_trie,tstar(1:n+1,k),n,alpha,beta,1e-8,1-1e-8,indic_Cprime);
            util_asym = temp' * vect_p(vect_y_asym,n) - alpha * theta_trie .* C_prime(vect_y_asym,...
                        beta, indic_Cprime);
            prop_gain(k) = mean((util_asym>util_lin));
            y_opt_asym(k) =  vect_y_asym*vect_weight;
            
        else
            % In this case the infinite approximation (r/n=y) is very good.
            theta_tr = fliplr(theta_trie)';
            if indic_Ftheta == 1
                Ftheta = proba_st_n(k) * exp(-a_st * (theta_tr.^(-b))) + ...
                        (1-proba_st_n(k)) * exp(-a_mov * (theta_tr.^(-b)));
                ftheta =b * (theta_tr.^(-b-1)).* (proba_st_n(k) * a_st * exp(-a_st * (theta_tr.^(-b))) + ...
                        (1-proba_st_n(k)) * a_mov * exp(-a_mov * (theta_tr.^(-b))));
                theta_F_f = theta_tr + Ftheta./ftheta;
            elseif indic_Ftheta ==2
                theta_F_f = theta_tr.*( 1 + (proba_st_n(k) * normcdf(log(theta_tr),a_st,b)+...
                        (1-proba_st_n(k))*normcdf(log(theta_tr),a_mov,b))./(proba_st_n(k)*...
                        normpdf(log(theta_tr),a_st,b)+(1-proba_st_n(k))*normpdf(log(theta_tr),a_mov,b)));
            else
                Ftheta = proba_st_n(k) * (1-exp(-a_st * (theta_tr.^b))) + ...
                        (1-proba_st_n(k)) * (1-exp(-a_mov * (theta_tr.^b)));
                ftheta = b * (theta_tr.^(b-1)).* (proba_st_n(k) * a_st * exp(-a_st * (theta_tr.^b)) + ...
                        (1-proba_st_n(k)) * a_mov * exp(-a_mov * (theta_tr.^b)));
                theta_F_f = theta_tr + Ftheta./ftheta;
            end
            yopt_asym =  inv_Cprime(beta, theta0, y0, delta1, lambda2./theta_F_f, indic_Cprime) ;
            tprime_y = lambda2 * theta_tr ./theta_F_f;        
            mat_diff_y = ones(n_s,1) * (yopt_asym - [0;yopt_asym(1:end-1)])';
            mat_integr = tril(ones(n_s)) .* mat_diff_y;
            t_y_asym = mat_integr * tprime_y +4.5;
            tstar(1:n+1,k)=[4.5;interp1([0;yopt_asym],[4.5;t_y_asym], (1/n:1/n:1)','linear','extrap')];
            yopt_asym2 = flipud(yopt_asym)';
            y_opt_asym(k) = yopt_asym2 * vect_weight;
            util_asym = tstar(1:n+1,k)' * vect_p(yopt_asym2,n) - alpha * theta_trie .* C_prime(yopt_asym2,...
                        beta, indic_Cprime);
            prop_gain(k) = mean((util_asym>util_lin));
            n_gd=50;
            approx=interp1([0;yopt_asym],[4.5;t_y_asym], (1/n_gd:1/n_gd:1)','linear','extrap');
            log_delta_t_y_asym = log(approx - [4.5;approx(1:end-1)]);
            vect_surplus(k)= - n* pr_t_opt2(log_delta_t_y_asym,alpha,beta,lambda2,n_gd,theta_trie,vect_weight,indic_Cprime)/n_gd;
        end
    end 

    if nargout > 1
        t_opt_asym = tstar;
    end

    mean_gain =  prop_n' * prop_gain;
    
    mean_y_opt_asym = prop_n' * y_opt_asym;
    surplus_asym = prop_n' * vect_surplus / n2moy;
    ratio_asym = surplus_actuel / surplus_asym ;

    % Without moral hazard
    mean_y_opt_WM = prop_n'*[repmat(y_opt_asym(4),4,1);y_opt_asym(5:end)];
    surplus_WM = prop_n'*[vect_surplus(4)*liste_n(1:4)/liste_n(4);vect_surplus(5:end)]./n2moy;
    ratio_WM = surplus_actuel / surplus_WM;

    %% Optimal contract with symmetric information

    % Sorting theta by descending order, so that y is sorted by ascending order

    indic_st=logical(fliplr(indic_stayers));

    theta_trie=fliplr(theta_trie);
    yopt_sym =  inv_Cprime(beta, theta0, y0, delta1, lambda2./theta_trie, indic_Cprime)';

    t_y_sym = zeros(n_s,1);

    for k=1:n_s
        t_y_sym(k) = 4.5 +  delta2 * integral(@(x) C_prime(x,beta, indic_Cprime),0,yopt_sym(k))/C_prime(yopt_sym(k),beta, indic_Cprime);
    end

    temp_sym = lambda2 * yopt_sym - t_y_sym;

    mean_y_opt_sym = p_st2 * mean(yopt_sym(indic_st)) + (1- p_st2) * mean(yopt_sym(~indic_st));

    surplus_sym = poids_st2 * mean(temp_sym(indic_st)) + (1-poids_st2)* mean(temp_sym(~indic_st)) ; 
    ratio_sym = surplus_actuel/surplus_sym ;

    %% Gathering all the results

    surplus =   [surplus_sym;surplus_WM;surplus_asym;surplus_actuel];
    ratio = [ratio_sym/ratio_WM;ratio_sym/ratio_asym;ratio_sym];
    moy_y = [mean_y_opt_sym;mean_y_opt_WM;mean_y_opt_asym;mean(y2)];
    res = [alpha; lambda1; lambda2; surplus; ratio; moy_y; mean_gain];
end

end