%% data loading and data management
clear, clc
data=csvread('../data_for_matlab.csv',1,1);
rng(1);

% Interviewers there in (2001 or 2002) and 2003
stayers = data((data(:,2)>0 & data(:,4)>0),:);
% Interviewers there only in 2001 or 2002
movers01 = data((data(:,2)>0 & data(:,4)==0),1:2);
% Interviewers there only in 2003
movers03 = data((data(:,2)==0 & data(:,4)>0),3:4);

n1_st = stayers(:,2);
r1_st = stayers(:,1);

n2_st = stayers(:,4);
r2_st = stayers(:,3);

n1_mov = movers01(:,2);
r1_mov = movers01(:,1);

n2_mov = movers03(:,2);
r2_mov = movers03(:,1);

delta1 = 20.2;
delta2 = 22.9;
E= delta1/delta2;


%% Figure 3: nonparametric sieve MLE estimation of F_{Y|X=x,S=s}
options = optimset('Display','final','MaxFunEvals',1e6,'MaxIter',100000);

a0 = sqrt(3)*[0;0;1;0];

a1 = fmincon(@(a) log_vrais_binom(a, r1_st, n1_st), a0, [],[],[],[],[],[],@contrainte_densite,options);

b1 = conv(a1,a1);
b1_int_st =  polyint(b1')';

a2 = fmincon(@(a) log_vrais_binom(a, r2_st, n2_st), a0, [],[],[],[],[],[],@contrainte_densite,options);

b2 = conv(a2,a2);
b2_int_st =  polyint(b2')';

% Graphs of F_{y_1} and F_{y_2}
grille = (0:1/999:1)';
F_y1_st = polyval(b1_int_st, grille);
F_y2_st = max(polyval(b2_int_st, grille),0);

subplot(1,2,1), plot(grille,F_y1_st,'k--',grille, F_y2_st,'k -')
axis([0 1 0 1])

set(gca, 'fontunits', 'points');
set(gca, 'fontsize',10);
set(gca, 'fontname', 'Times');
legend('2001-2002, stayers','2003, stayers','Location','NorthWest')
legend_handle = legend;
set(legend_handle, 'Box', 'off')

% Nonparametric sieve MLE estimation of F_{y_1|mov} 
a0 = sqrt(3)*[0;1;0];
a1 = fmincon(@(a) log_vrais_binom(a, r1_mov, n1_mov), a0, [],[],[],[],[],[],@contrainte_densite,options);
b1 = conv(a1,a1);
b1_int_mov =  polyint(b1')';

% Nonparametric sieve MLE estimation of F_{y_2|mov} 
a2 = fmincon(@(a) log_vrais_binom(a, r2_mov, n2_mov), a0, [],[],[],[],[],[],@contrainte_densite,options);
b2 = conv(a2,a2);
b2_int_mov =  polyint(b2')';

F_y1_mov = polyval(b1_int_mov, grille);
F_y2_mov = polyval(b2_int_mov, grille);

subplot(1,2,2), plot(grille,F_y1_mov,'k--',grille, F_y2_mov,'k -')
axis([0 1 0 1])

set(gca, 'fontunits', 'points');
set(gca, 'fontsize',10);
set(gca, 'fontname', 'Times');
legend('2001-2002, movers','2003, movers','Location','NorthWest')
legend_handle = legend;
set(legend_handle, 'Box', 'off')

%% Test of no selection (cf. Section 4.1)

n_gr = length(grille);
qq_st = zeros(n_gr,1);
% Diff computes F_{y1|S=0} o F_{y2|S=1}^{-1} o F_{y2|S=1} - F_{y2|S=0}
Diff =  zeros(n_gr,1);
for b=1:n_gr
    % We first find u=F_{y2|S=1}^{-1} o F_{y2|S=1}(b), by solving
    % F_{y2|S=1}(u) - F_{y2|S=1}(b)=0 => root of the polynomial  F_{y2|S=1}
    % -  F_{y2|S=1}(b)
    p=  [b1_int_st(1:end-1)', - F_y2_st(b)];
    qq_st(b) = sum(roots(p).* (imag(roots(p))==0) .* (roots(p)<=1.0001) .* (roots(p)>=-0.0001));
    Diff(b) = polyval(b1_int_mov,qq_st(b)) - F_y2_mov(b);    
end

test_no_sel = max(abs(Diff));

nb_boot = 1e3;
test_boot=test_selection_boot(data,b1_int_st, b2_int_st,b1_int_mov,grille,nb_boot);

p_value = mean((test_boot>test_no_sel));

disp 'Statistic and p-value for the selection test'
[test_no_sel p_value]

%%  Figure 4, part 1: Nonparametric estimates of F_theta(theta_n) (stayers) and C'(y_n)

% First, estimation of K and the sequence (y_{-k})_{k in K} (in decreasing 
% order and (theta_{-k}) (in increasing order).

% Initial points (normalization)
theta0 =  1 ;
% y0 is approximately the median of F_{y1}
y0 = 0.8;

% Estimation of the support of Y1
tau_L = 0.05;
borne_inf= interp1q(F_y2_st,grille,tau_L);
borne_sup = interp1q(F_y1_st,grille,1-tau_L);

grid_y = y0;

y_temp = y0;
k=0;

while y_temp < borne_sup & k>-50
    fdr_temp = polyval(b1_int_st, y_temp);
    p = [b2_int_st(1:end-1)', b2_int_st(end)-fdr_temp];
    y_temp = sum(roots(p).* (imag(roots(p))==0) .* (roots(p)<1) .* (roots(p)>0));
    grid_y=[y_temp;grid_y];
    k=k-1;
end

fdr_temp = polyval(b1_int_st, y_temp);
p = [b2_int_st(1:end-1)' b2_int_st(end)-fdr_temp];
hors_grid = sum(roots(p).* (imag(roots(p))==0) .* (roots(p)<1) .* (roots(p)>0));

nb_g = -k;

y_temp = y0;
k=0;

while y_temp > borne_inf & k<50
    fdr_temp = polyval(b2_int_st, y_temp);
    p = [b1_int_st(1:end-1)', b1_int_st(end)-fdr_temp];
    y_temp = sum(roots(p).* (imag(roots(p))==0) .* (roots(p)<1) .* (roots(p)>0));
    grid_y=[grid_y;y_temp];
    k=k+1;
end

nb_d = k;
% The grid of (theta_k) is simple since theta_k = E^k theta0
grid_theta = theta0 * (E.^((nb_g:-1:-nb_d)'));

% Number of points
disp 'Number of points that we recover'
taille_tot = nb_g + nb_d + 1

% Nonparametric estimates of F_theta(theta_n) for stayers and C'(y_n)

Ftheta_st = 1-polyval(b1_int_st, grid_y);
C_prime_non_param = delta1 ./ grid_theta ;

% Estimation of F_theta(theta_n) on the whole population (in 2003)

% Proportion of stayers
p_st = 1/(1+length(n2_mov)/length(n2_st));

Ftheta_mov1 = 1-polyval(b1_int_mov,grid_y);
Ftheta_mov2 = 1-polyval(b2_int_mov,[hors_grid;grid_y(1:end-1)]);

% Nonparametric estimate of F_theta
Ftheta_non_param = p_st * Ftheta_st + (1-p_st) * Ftheta_mov2;

%% Figure 4, part 2: nonparametric estimated bounds and confidence 
% intervals (by bootstrap) on F_theta and C'

pas = 1e-5;
grid_theta_inf = grid_theta - pas;
grid_theta_sup = grid_theta + pas;
grid_theta_bis = sort([1e-6;grid_theta_inf;grid_theta;grid_theta_sup;1e6])';
nb_pts = length(grid_theta_bis);

temp = grid_theta * ones(1,nb_pts) ; 
indic1 = (temp <= ones(taille_tot,1)* grid_theta_bis);
indic2 = (temp >= ones(taille_tot,1)* grid_theta_bis);
ind1 =  1 + sum(indic1)' ;
ind2 =  1 + taille_tot - sum(indic2)' ;

Fth_bis = [0;Ftheta_non_param];
Fth_ter = [Ftheta_non_param;1];

Ftheta_inf = Fth_bis(ind1);
Ftheta_sup = Fth_ter(ind2);

y_bis = [grid_y;0];
y_ter = [1;grid_y];

yinf = y_bis(ind2);
ysup = y_ter(ind1);

nb_draws = 5e2 ;
res = bootstrp(nb_draws, @(dt) estim_sequence(dt, grid_theta_bis, E, tau_L),data); 

ICy_inf=quantile(res(:,1:nb_pts),0.025);
ICy_sup=quantile(res(:,nb_pts+1:2*nb_pts),0.975);
IC_Finf=quantile(res(:,2*nb_pts+1:3*nb_pts),0.025);
IC_Fsup=quantile(res(:,3*nb_pts+1:4*nb_pts),0.975);
 
scrsz = get(0,'ScreenSize');
figure('Position',[1 scrsz(4)/2 scrsz(3)/2 scrsz(4)/5])

subplot(1,2,1) 
line(grid_theta_bis, Ftheta_sup,'Color',[0 0 0],'LineWidth',1)
line(grid_theta_bis, Ftheta_inf,'Color',[0.6 0.6 0.6],'LineWidth',1)
line(grid_theta_bis, IC_Finf, 'Color',[0 0 0], 'LineStyle',':','LineWidth',0.5)
line(grid_theta_bis, IC_Fsup, 'Color',[0 0 0], 'LineStyle',':','LineWidth',0.5)
title('Bounds on F_{\theta|X=2}','fontunits', 'points', 'fontsize',9, 'fontname', 'Times');

axis([0 3 0 1])
legend('Upper bound','Lower bound','95% CI','Location','SouthEast')
legend_handle = legend;
set(legend_handle, 'Box', 'off')
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',7);
set(gca, 'fontname', 'Times');

ordonnees = delta1./grid_theta_bis;
subplot(1,2,2) 
line(yinf, ordonnees, 'Color',[0 0 0],'LineWidth',1)
line(ysup,ordonnees, 'Color',[0.6 0.6 0.6],'LineWidth',1)
line(ICy_inf,ordonnees, 'Color',[0 0 0], 'LineStyle',':','LineWidth',0.5)
line(ICy_sup,ordonnees, 'Color',[0 0 0], 'LineStyle',':','LineWidth',0.5)
title('Bounds on C''','fontunits', 'points', 'fontsize',9, 'fontname', 'Times');

axis([0 1 0 90])
legend('Upper bound','Lower bound','95% CI','Location','NorthWest')
legend_handle = legend;
set(legend_handle, 'Box', 'off')
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',7);
set(gca, 'fontname', 'Times');

%% Table 6: overidentification test of Assumption 3
% Bootatrap GMM overidentication test based on E[1{N=n}(1{R=r} - p_{r,n})|X,S]=0, 
% where p_{r,n}=Pr(R=r|N=n, X, S) according to the polynomial model.

% Number of bootstrap replications
B=1e3;

% Stayers 2001-2002
% Last but one argument=(degree of the sieve polynomial)+1
[~,pval_st1]=test_boot_specif_MLE(r1_st, n1_st, 4, B);

% Stayers 2003
[~,pval_st2]=test_boot_specif_MLE(r2_st, n2_st, 4, B);
 
% Movers 2001-2002
[~,pval_mov1]=test_boot_specif_MLE(r1_mov, n1_mov, 3, B);

% Movers 2003
[~,pval_mov2]=test_boot_specif_MLE(r2_mov, n2_mov, 3, B);

disp 'Table 6 (p-values)'
[pval_st1,pval_st2,pval_mov1,pval_mov2]

%% Figure 5: choice of the parametric form on F_theta 

log_theta = log(grid_theta);
log_log_F = -log(-log(Ftheta_st));
norminvF = norminv(Ftheta_st);
log_log_S = log(-log(1-Ftheta_st));

M1 = [ones(taille_tot,1), log_theta];
linear1 = M1 * (M1\log_log_F);
linear2 = M1 * (M1\norminvF);
linear3 = M1 * (M1\log_log_S);

R21= 1-var(log_log_F - linear1)/var(log_log_F);
R22= 1-var(norminvF - linear2)/var(norminvF);
R23= 1-var(log_log_S - linear3)/var(log_log_S);

scrsz = get(0,'ScreenSize');
figure('Position',[1 scrsz(4)/2 scrsz(3)/2 scrsz(4)/5])

subplot(1,3,1), plot(log_theta, log_log_F,'k +', log_theta, linear1, 'k--');
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',6);
set(gca, 'fontname', 'Times');
string_R21=strcat('$R^2=',num2str(round(R21,3)),'$');
annotation(gcf,'textbox',[0.26 0.13 1 0.1],'String',{string_R21},'Interpreter','latex','FitHeightToText','on','LineStyle','none','fontsize',8);
title('Fit for the Frechet','FontName','Times','FontWeight','normal','FontSize',8)

subplot(1,3,2), plot(log_theta, norminvF,'k +', log_theta, linear2, 'k--');
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',6);
set(gca, 'fontname', 'Times');
string_R22=strcat('$R^2=',num2str(round(R22,3)),'$');
annotation(gcf,'textbox',[0.54 0.13 1 0.1],'String',{string_R22},'Interpreter','latex','FitHeightToText','on','LineStyle','none','fontsize',8);
title('Fit for the Lognormal','FontName','Times','FontWeight','normal','FontSize',8)

subplot(1,3,3), plot(log_theta, log_log_S,'k +', log_theta, linear3, 'k--'); 
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',6);
set(gca, 'fontname', 'Times');
string_R23=strcat('$R^2=',num2str(round(R23,3)),'$');
annotation(gcf,'textbox',[0.82 0.13 1 0.1],'String',{string_R23},'Interpreter','latex','FitHeightToText','on','LineStyle','none','fontsize',8);
title('Fit for the Weibull','FontName','Times','FontWeight','normal','FontSize',8)


%% Figure 6: choice of the parametric form on C'
transf_y1 = log(-log(1-grid_y)) ;
transf_y2 = log(grid_y./(1-grid_y)) ;
transf_y3 = log(exp(grid_y./(1-grid_y))-1);
log_Cprime = log(C_prime_non_param);

M1 = [ones(taille_tot,1), transf_y1];
linear1 = M1 * (M1\log_Cprime);
R21= 1-var(log_Cprime - linear1)/var(log_Cprime);
M2 = [ones(taille_tot,1), transf_y2];
linear2 = M2 * (M2\log_Cprime);
R22= 1-var(log_Cprime - linear2)/var(log_Cprime);
M3 = [ones(taille_tot,1), transf_y3];
linear3 = M3 * (M3\log_Cprime);
R23= 1-var(log_Cprime - linear3)/var(log_Cprime);

scrsz = get(0,'ScreenSize');
figure('Position',[1 scrsz(4)/2 scrsz(3)/2 scrsz(4)/5])

subplot(1,3,1), plot(transf_y1, log_Cprime,'k +', transf_y1, linear1, 'k--')
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',8);
set(gca, 'fontname', 'Times');
string_R21=strcat('$R^2=',num2str(round(R21,3)),'$');
annotation(gcf,'textbox',[0.26 0.13 1 0.1],'String',{string_R21},'Interpreter','latex','FitHeightToText','on','LineStyle','none','fontsize',8);
title('Fit for $\phi(x)=\ln(1+x)$','Interpreter','Latex','FontName','Times','FontWeight','normal','FontSize',8)
subplot(1,3,2), plot(transf_y2, log_Cprime,'k +', transf_y2, linear2, 'k--')
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',8);
set(gca, 'fontname', 'Times');
string_R22=strcat('$R^2=',num2str(round(R22,3)),'$');
annotation(gcf,'textbox',[0.54 0.13 1 0.1],'String',{string_R22},'Interpreter','latex','FitHeightToText','on','LineStyle','none','fontsize',8);
title('Fit for $\phi(x)=x$','Interpreter','Latex','FontName','Times','FontWeight','normal','FontSize',8)
subplot(1,3,3), plot(transf_y3, log_Cprime,'k +', transf_y3, linear3, 'k--') 
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',8);
set(gca, 'fontname', 'Times');
string_R23=strcat('$R^2=',num2str(round(R23,3)),'$');
annotation(gcf,'textbox',[0.82 0.13 1 0.1],'String',{string_R23},'Interpreter','latex','FitHeightToText','on','LineStyle','none','fontsize',8);
title('Fit for $\phi(x)=\exp(x)-1$','Interpreter','Latex','FontName','Times','FontWeight','normal','FontSize',8)


%% Table 7: Parametric estimation of F_\theta and C'

% indic_Ftheta: 1:Frechet, 2: lognormal, 3: Weibull
% indic_Cprime: 1:phi=ln(1+y), 2:phi=y, 3:phi=exp(y)-1
% Prefereed specif: lognormal and phi(y)=ln(1+y)

indic_Ftheta = 2;
indic_Cprime = 1;

% Number of points used to approximate the integral wrt theta
n_s = 1000;

% Unconstrained estimation (first column of Table 7)
nb_param_theta = 4;
[res,mat_var]=estimation_param(data, nb_param_theta, theta0, y0, delta1, delta2, n_s, indic_Ftheta, indic_Cprime);

% Parameters
vect_param = res;

% Computation of alpha and lambda
alpha_lambda = estimation_surplus_et_al(data, vect_param, theta0, y0, ...
                    delta1, delta2, n_s, indic_Ftheta, indic_Cprime, [], 0);               
                
% Standard errors
eps = 1e-3;
nb_param = length(vect_param);
jac_unc = zeros(length(alpha_lambda), nb_param);

for k=1:nb_param
    vect_param_modif = vect_param + eps * ((1:nb_param)'==k);
    alpha_lambda_modif=estimation_surplus_et_al(data, vect_param_modif, theta0, ...
                y0, delta1, delta2, n_s, indic_Ftheta, indic_Cprime, [],0);
    jac_unc(:,k) = (alpha_lambda_modif - alpha_lambda)/eps;
end

et_alpha_lambda = sqrt(diag(jac_unc * mat_var * jac_unc'));
et_unc=[sqrt(diag(mat_var));et_alpha_lambda];

res_unc=[res;alpha_lambda];

disp 'Table 7, 1st column (order: a3, a1, a2, b, beta, alpha, lambda1,lambda2)'
[res_unc, et_unc]
                
% Selection test (test of a1=a2 in the paper)
J1=[0,-1,1,0,0];
var_diff1 = J1*mat_var*J1';
diff1 = J1 * res;
test1 = diff1^2/var_diff1;
p_value1 = 1-chi2cdf(test1,1)

% Constrained estimation (2nd column of Table 7)
nb_param_theta = 3;
[res,mat_var]=estimation_param(data, nb_param_theta, theta0, y0, delta1, ...
                                delta2, n_s, indic_Ftheta, indic_Cprime);

vect_param=res;

% Computation of alpha and lambda
alpha_lambda = estimation_surplus_et_al(data, vect_param, theta0, y0, ...
                    delta1, delta2, n_s, indic_Ftheta, indic_Cprime, [], 0);               
                
% Standard errors
eps = 1e-3;
nb_param = length(vect_param);
jac_con = zeros(length(alpha_lambda), nb_param);

for k=1:nb_param
    vect_param_modif = vect_param + eps * ((1:nb_param)'==k);
    alpha_lambda_modif=estimation_surplus_et_al(data, vect_param_modif, theta0, ...
                y0, delta1, delta2, n_s, indic_Ftheta, indic_Cprime, [],0);
    jac_con(:,k) = (alpha_lambda_modif - alpha_lambda)/eps;
end

et_alpha_lambda = sqrt(diag(jac_con * mat_var * jac_con'));
et_con=[sqrt(diag(mat_var));et_alpha_lambda];

res_con=[res;alpha_lambda];

disp 'Table 7, 2nd column (order: a3, a1, b, beta, alpha, lambda1,lambda2)'
[res_con, et_con]

%% Table 8

% Initial values for the contracts with n=2, 3, 4
T2=[4.5000;4.5280; 23.8457];
T3=[4.5000;4.7001;6.9557;24.7156];
T4=[4.5000;4.5538;6.5605;10.0879; 24.7313];
t0 =[[T2;0;0], [T3;0],T4];
[surplus_et_al,tstar] = estimation_surplus_et_al(data, vect_param, theta0, ...
                            y0, delta1, delta2, n_s, indic_Ftheta, indic_Cprime,t0,1);

                        
eps = 1e-3;
nb_param = length(vect_param);
jac_surplus = zeros(length(surplus_et_al), nb_param);

for k=1:nb_param
    vect_param_modif = vect_param + eps * ((1:nb_param)'==k);
    surplus_et_al_modif=estimation_surplus_et_al(data, vect_param_modif, theta0, ...
                y0, delta1, delta2, n_s, indic_Ftheta, indic_Cprime, t0,1);
    jac_surplus(:,k) = (surplus_et_al_modif - surplus_et_al)/eps;
end

mat_var_surplus = jac_surplus * mat_var * jac_surplus';
et=sqrt(diag(mat_var_surplus));

disp 'Point estimates and standard errors in Table 8'
% By columns, then by row. 
[surplus_et_al(4:end-1), et(4:end-1)]

disp 'Proportion of interv. who gain moving to the optimal asym. contract'
surplus_et_al(end)

disp 'Gain of no moral hazard (in percent)'
1e2*(surplus_et_al(5)/surplus_et_al(6) -1)


%% Figure 7

grille_y= (0:0.01:1);

T1=[4.5;22.9+4.5];
poly1 = T1'*vect_p(grille_y,1);
poly2 = tstar(1:3,1)'*vect_p(grille_y,2) ;
poly3=tstar(1:4,2)'*vect_p(grille_y,3);
poly4=tstar(1:5,3)'*vect_p(grille_y,4);
poly50=tstar(:,end)'*vect_p(grille_y,53);

h = plot(grille_y,poly1,'k :',grille_y,poly2,'k--',grille_y,poly3,'k-.',grille_y,poly50,'k');
set(h(1),'LineWidth',1.5);
set(gca, 'fontunits', 'points');
set(gca, 'fontsize',10);
set(gca, 'fontname', 'Times');
legend('$\widetilde{t}_1$','$\widetilde{t}_2$','$\widetilde{t}_3$','$\widetilde{t}_\infty$','Location','NorthWest')
legend_handle = legend;
set(legend_handle, 'Box', 'off','interpreter','latex','fontsize',10)


%% Table 9: using information on interviewers
X=data(:,5:end);
X(X== -99) = NaN;
data_fin=[data(:,1:4),X];

load 'data_with_covariates'
data_X = double(st_dataset);

n_s = 1e3;
vect_unif = halton(n_s,3)';

nu0 = [zeros(7,1);log(0.4);0.2];
options = optimset('GradObj','off','LargeScale','off');

[temp_mle,~,~,~,~, hessian] = fminunc(@(nu) log_vrais_parametrique_X(nu, theta0, y0, delta1, ...
                delta2, vect_unif, data_fin, 1), nu0, options);
            
a0  = temp_mle(1);
a = temp_mle(2:end-2);
sigma= exp(temp_mle(end-1));
beta = exp(temp_mle(end));
coeffs= [a0;a;sigma;beta];
jac = diag([ones(length(coeffs)-2,1);sigma;beta]);

V_init= inv(hessian)/size(data_fin,1);
mat_var_coeff = jac * V_init * jac';

% First column
disp 'Table 9: first column'
[coeffs, sqrt(diag(mat_var_coeff))]

% R2 of the model (first column)
X2 = data_fin(:,11:end);
index = X2 * a;
index = index(~isnan(index));
disp 'Table 9: R2, first column'
R2 = var(index)/(var(index)+sigma^2)

% Estimation with only experience and urban/rural
data_fin = [data_X(:,1:7), data_X(:,11:13)];

nu0 = [zeros(4,1);log(0.4);0.2];
            
options = optimset('GradObj','off','LargeScale','off');

[temp_mle,~,~,~,~, hessian] = fminunc(@(nu) log_vrais_parametrique_X(nu, theta0, y0, delta1, ...
                delta2, vect_unif, data_fin, 1), nu0, options);
            
a0  = temp_mle(1);
a = temp_mle(2:end-2);
sigma= exp(temp_mle(end-1));
beta = exp(temp_mle(end));
coeffs= [a0;a;sigma;beta];
jac = diag([ones(length(coeffs)-2,1);sigma;beta]);

V_init= inv(hessian)/size(data_fin,1);
mat_var_coeff = jac * V_init * jac';

% Second column
disp 'Table 9: second column'
[coeffs, sqrt(diag(mat_var_coeff))]

% R2 of the model (second column)
X2 = data_fin(:,end-size(a,1)+1:end);
index = X2 * a;
index = index(~isnan(index));
disp 'R2, second column'
R2 = var(index)/(var(index)+sigma^2)

%% Test of independence b/w n and theta (Appendix B.1)

data1 = data(data(:,2)>0 & data(:,2)<35,1:2);
data2 = data(data(:,4)>0 & data(:,4)<35,3:4);

r1=data1(:,1);
n1=data1(:,2);
r2=data2(:,1);
n2=data2(:,2);

[tbl1,~,~,labels]=crosstab(n1,r1);
temp=str2double(labels);
vect_K1 = temp(:,1)';
% Construction of a proper database with 0 if needed
for_db1 = zeros(length(vect_K1), max(vect_K1)+1);

for j=1:max(vect_K1)+1
    if sum(temp(:,2)==j-1)>0
        for_db1(:,j) = tbl1(:,(temp(:,2)==j-1));
    end
end
    
db1 = -99*ones(length(vect_K1), max(vect_K1)+2);
i=1;
for K=vect_K1
    db1(i,1:K+1)=for_db1(i,1:K+1);
    i=i+1;
end

[tbl2,~,~,labels]=crosstab(n2,r2);
temp=str2double(labels);
vect_K2 = temp(:,1)';
% Construction of a proper database with 0 if needed
for_db2 = zeros(length(vect_K2), max(vect_K2)+1);

for j=1:max(vect_K2)+1
    if sum(temp(:,2)==j-1)>0
        for_db2(:,j) = tbl2(:,(temp(:,2)==j-1));
    end
end
    
db2 = -99*ones(length(vect_K2), max(vect_K2)+2);
i=1;
for K=vect_K2
    db2(i,1:K+1)=for_db2(i,1:K+1);
    i=i+1;
end

% 2001-2002 survey
xp_init= stat_test_n_indep_theta(db1, vect_K1);
disp 'Test of indep. b/w n and theta, p-value (2001-2002)'
pvalue1=test_n_indep_theta(db1,vect_K1,5e2,xp_init)
% 0.92

% 2003 survey
xp_init= stat_test_n_indep_theta(db2, vect_K2);
disp 'Test of indep. b/w n and theta, p-value (2003)'
pvalue2=test_n_indep_theta(db2,vect_K2,5e2,xp_init)
% 0.70


%% Figure 8: alternative estimation of $F_{Y|X,S=1}
% Estimation based on DH-R (2017), which uses only the first moment 
% conditions on Y|X. 
% Constraints on the support points: increasing sequence, positive and <= 1
% Also: constraints on the probabilities (>0)
L= floor(max(n1_st)/2)+1;
Ad = [ [[-1,zeros(1,L-1)];eye(L) - diag(ones(L-1,1),1)] zeros(L+1,L); zeros(L,L) (-eye(L))]; 
bd = [[-1e-6;-2e-2 * ones(L-1,1);1]; -1e-6*ones(L,1)];
% Constraint on the probabilities: sum to 1
Adeq = [zeros(1,L) ones(1,L)];
bdeq = 1;    
supp_proba0 = [ones(L,1)/L;(1/(L+1):1/(L+1):(L)/(L+1))'] ;
opts = optimset(optimset('fmincon'),'Display','off','MaxIter',2000,...
                    'MaxFunEvals',50000,'TolCon',1e-7,'Algorithm','sqp');

supp_proba_max_vrais1 = fmincon(@(supp_proba) estimation_DH_Rath(supp_proba,...
    r1_st, n1_st), supp_proba0, Ad, bd, Adeq, bdeq,[],[],[],opts);
supp_proba_max_vrais2 = fmincon(@(supp_proba) estimation_DH_Rath(supp_proba,...
    r2_st, n2_st), supp_proba0, Ad, bd, Adeq, bdeq,[],[],[],opts);

support1 = supp_proba_max_vrais1(1:end/2);
proba1 = supp_proba_max_vrais1(end/2+1:end);

support2 = supp_proba_max_vrais2(1:end/2);
proba2 = supp_proba_max_vrais2(end/2+1:end);

% Comparison with the initial estimates of F_{Y|X,S=1} 
F_y1_DHR = (repmat(support1,1,1e3)<=repmat(grille',L,1))' * proba1;
F_y2_DHR = (repmat(support2,1,1e3)<=repmat(grille',L,1))' * proba2;

delete(findall(gcf,'Tag','stream'));
subplot(1,2,1) 
plot(grille,F_y1_st,'k--',grille, F_y1_DHR,'k -')
set(gca, 'fontname', 'Times');
legend('Polynomial estimate','DH-R estimate','Location','NorthWest')
legend_handle = legend;
set(legend_handle, 'Box', 'off')
title('F_{Y|X=1,S=1}','fontunits', 'points', 'fontsize',9, 'fontname', 'Times');

subplot(1,2,2) 
plot(grille,F_y2_st,'k--',grille, F_y2_DHR,'k -')
set(gca, 'fontname', 'Times');
legend('Polynomial estimate','DH-R estimate','Location','NorthWest')
legend_handle = legend;
set(legend_handle, 'Box', 'off')
title('F_{Y|X=2,S=1}','fontunits', 'points', 'fontsize',9, 'fontname', 'Times');