% This function computes the GMM overidentification test of the model based
% on a polynomial specification for F_y (for a given polynomial,
% corresponding to vector a). r=vector of (R_i), n=vector of (N_i), W = GMM
% weighting matrix.

function res = stat_test_specif_MLE(a,r,n,W)

b = conv(a,a);
% degree of the polynomial corresponding to F_y
K_L = length(b)-1;

[~,~,~,labels]=crosstab(n,r);
temp=str2double(labels);
% All possible values of n.
vect_n = temp(~isnan(temp(:,1)),1)';

% index of the moment condition
ind = 1;
% vector of the moment conditions
moment = zeros(size(W,1),1);

for i=vect_n
    % We look at moments of the kind E[1{N=n}(1{R=r} - p_{r,n})|X,S], with
    % 0<r<=n. We do not include 0 because of moment redundancy.
   for j=1:i
       % Probability Pr(R=j|N=i,X=x,S=s) according to the polynomial model
       p_ij = nchoosek(i,j)*(beta(j+(0:K_L)+1,i-j+1) * flipud(b));
       moment(ind) = mean((n==i).*((r==j) - p_ij));
       ind = ind + 1;
   end
end

res = length(n) * moment' * W * moment;

end